$ifndef _DS1307_INC_
$define _DS1307_INC_
'
' Routines to access a DS1307 Real Time Clock/Calendar via a software I2C bus
' Written by L.Johnson 08-01-16
'
' Declare the default pins used for busin and busout
'
#ifnSym __SDA_Port
    Declare SDA_Pin = PORTB.0           ' SDA connected to this pin
#endIfSym
#ifnSym __SCL_Port
    Declare SCL_Pin = PORTB.1           ' SCL connected to this pin
#endIfSym
'
' Time variables
'
    Dim RTC_Second As Byte              ' Second (0..59)
    Dim RTC_Minute As Byte              ' Minute (0..59)
    Dim RTC_Hour   As Byte              ' Hour    (0..11 or 0..23)
'
' Date variables
'
    Dim RTC_Day       As Byte           ' Date  (0..31)
    Dim RTC_Month     As Byte           ' Month (1..12)
    Dim RTC_Year      As Byte           ' Year  (0..99)
    Dim RTC_DayOfWeek As Byte           ' Day of the week (1..7)
    Dim RTC_DayName As String * 9       ' Holds the name of the day
    Dim RTC_MonthName As String * 9     ' Holds the name of the month
'
' Parameter variables
'
    Dim DS1307_bValue As Byte
    Dim DS1307_bResult As Byte
'
' DS1307 register addresses
'
     $define cRTC_DEV_ADDR  $D0         ' DS1307 device Address
     $define cRTC_TIME      $00         ' Address for time
     $define cRTC_WDAY      $03         ' Address for day of week number
     $define cRTC_DATE      $04         ' Address for date
     $define cRTC_CONTROL   $07         ' Address for control byte
     $define cRTC_NVRAM     $08         ' NVRAM start
     $define cRTC_NVRAM_MAX $38         ' NVRAM address max (56 decimal)
'
' Values - used in calls to DS1307_ReadControl() and DS1307_WriteControl()...
'
     $define cRTC_SQWE       $10        ' Square Wave enable
     $define cRTC_SQWF_1HZ   $00        ' Square Wave frequency  1Hz
     $define cRTC_SQWF_4KHz  $01        ' Square Wave frequency  4.096KHz
     $define cRTC_SQWF_8KHz  $02        ' Square Wave frequency  8.192KHz
     $define cRTC_SQWF_32KHz $03        ' Square Wave frequency 32.768KHz
     $define cRTC_SQWOUT_H   $80        ' Set Square Wave output to high when disabled

$ifndef False
    $define False 0
$endif
$ifndef True
    $define True 1
$endif
  
'----------------------------------------------------------------------------    
    GoTo _DS1307_Main                   ' Jump over the routines
'----------------------------------------------------------------------------
' Convert a packed BCD (Binary Coded Decimal) value to a decimal value
' Input     : pValue holds the BCD value to convert to decimal
' Output    : pResult holds the decimal value
' Notes     : None
'
$define DS1307_BCDToDec(pValue, pResult) '
    DS1307_bValue = pValue               '
    GoSub _DS1307_BCDToDec               '
    pResult = DS1307_bResult
    
_DS1307_BCDToDec:
    DS1307_bResult = (DS1307_bValue >> 4) * 10
    DS1307_bResult = DS1307_bResult + (DS1307_bValue & $0F)
    Return

'----------------------------------------------------------------------------
' Convert a decimal value to packed BCD (Binary Coded Decimal) value
' Input     : pValue holds the decimal value to convert to BCD
' Output    : pResult holds the packed BCD value
' Notes     : None
'
$define DS1307_DecToBCD(pValue, pResult) '
    DS1307_bValue = pValue               '
    GoSub _DS1307_DecToBCD               '
    pResult = DS1307_bResult

_DS1307_DecToBCD:
    DS1307_bResult = ((DS1307_bValue / 10) << 4)
    DS1307_bResult = DS1307_bResult | (DS1307_bValue // 10)
    Return

'----------------------------------------------------------------------------
' Read a DS1307 register ($00..$3F)
' Input     : pAddress holds the address of the register
' Output    : pResult holds the byte read from the register
' Notes     : None
'
$define DS1307_ReadRegister(pAddress, pResult) BusIn cRTC_DEV_ADDR, pAddress,[pResult]

'----------------------------------------------------------------------------
' Write to a DS1307 register ($00..$3F)
' Input     : pAddress holds the address of the register
'           : pData holds the byte to write to the register
' Output    : None
' Notes     : None
'
$define DS1307_WriteRegister(pAddress, pData) BusOut cRTC_DEV_ADDR, pAddress,[pData]

'----------------------------------------------------------------------------
' Read the RTC control byte ($07)
' Input     : None
' Output    : pResult holds the byte read from the register
' Notes     : See the control constants cRTC_SQWXXXX above
'
$define DS1307_ReadControl(pResult) pResult = DS1307_ReadRegister(cRTC_CONTROL)

'----------------------------------------------------------------------------
' Write the RTC Control byte ($07)
' Input     : pData holds the byte to write to the register
' Output    : None
' Notes     : See the control constants cRTC_SQWXXXX above
'
$define DS1307_WriteControl(pData) BusOut cRTC_DEV_ADDR, cRTC_CONTROL,[pData]

'----------------------------------------------------------------------------
' Write a byte to Non-Volatile RAM
' Input     : pAddress holds the Non-Volatile RAM address
'           : pData holds the byte to write to Non-Volatile RAM
' Output    : None
' Notes     : The RAM is physically located at
'           : $08..$3F - this routine uses address $00..$37 (56 bytes)
'
$define DS1307_WriteByte(pAddress, pData) '
    If pAddress < cRTC_NVRAM_MAX Then     '
        DS1307_WriteRegister(pAddress + cRTC_NVRAM, pData) '
    EndIf

'----------------------------------------------------------------------------
' Read a byte from Non-Volatile RAM
' Input     : pAddress holds the Non-Volatile RAM address
' Output    : pResult holds the byte read from NV RAM
' Notes     : The RAM is physically located at
'           : $08..$3F - this routine uses address $00..$37 (56 bytes)
'
$define DS1307_ReadByte(pAddress, pResult) pResult = DS1307_ReadRegister(pAddress + cRTC_NVRAM)

'----------------------------------------------------------------------------
' Read the Time
' Input     : None
' Output    : Variable RTC_Second holds the seconds value (in decimal)
'           : Variable RTC_Minute holds the minutes value (in decimal)
'           : Variable RTC_Hour holds the hours value (in decimal)
' Notes     : None
'
$define DS1307_ReadTime() GoSub _DS1307_ReadTime

_DS1307_ReadTime:
    BusIn cRTC_DEV_ADDR, cRTC_TIME, [RTC_Second, RTC_Minute, RTC_Hour]

    DS1307_BCDToDec(RTC_Second, RTC_Second)       ' Convert the Seconds value to decimal
    DS1307_BCDToDec(RTC_Minute, RTC_Minute)       ' Convert the Minutes value to decimal
    DS1307_BCDToDec(RTC_Hour, RTC_Hour)           ' Convert the Hours value to decimal
    Return

'----------------------------------------------------------------------------
' Read the Date
' Input     : None
' Output    : Variable RTC_DayOfWeek holds the week day value (in decimal)
'           : Variable RTC_Day holds the days value (in decimal)
'           : Variable RTC_Month holds the months value (in decimal)
'           : Variable RTC_Year holds the years value (in decimal)
' Notes     : None
'
$define DS1307_ReadDate() GoSub _DS1307_ReadDate

_DS1307_ReadDate:
    BusIn cRTC_DEV_ADDR, cRTC_WDAY, [RTC_DayOfWeek, RTC_Day, RTC_Month, RTC_Year]

    DS1307_BCDToDec(RTC_DayOfWeek, RTC_DayOfWeek)     ' Convert the WeekDay value to decimal
    DS1307_BCDToDec(RTC_Day, RTC_Day)                 ' Convert the Days value to decimal
    DS1307_BCDToDec(RTC_Month, RTC_Month)             ' Convert the Months value to decimal
    DS1307_BCDToDec(RTC_Year, RTC_Year)               ' Convert the Years value to decimal
    Return

'----------------------------------------------------------------------------
' Set the Time
' Input     : pHour holds the hours value (in decimal)
'           : pMinute holds the minutes value (in decimal)
'           : pSecond holds the seconds value (in decimal)
' Output    : None
' Notes     : None
'
$define DS1307_WriteTime(pHour, pMinute, pSecond) '
    RTC_Hour = pHour                              '
    RTC_Minute = pMinute                          '
    RTC_Second = pSecond                          '
    GoSub _DS1307_WriteTime

_DS1307_WriteTime:
    DS1307_DecToBCD(RTC_Second, RTC_Second)       ' Convert the Seconds value to decimal
    DS1307_DecToBCD(RTC_Minute, RTC_Minute)       ' Convert the Minutes value to decimal
    DS1307_DecToBCD(RTC_Hour, RTC_Hour)           ' Convert the Hours value to decimal
    
    BusOut cRTC_DEV_ADDR, cRTC_TIME, [RTC_Second, RTC_Minute, RTC_Hour]
    Return
    
'----------------------------------------------------------------------------
' Set the Date
' Input     : pDayOfWeek holds the week day value (in decimal)
'           : pDay holds the days value (in decimal)
'           : pMonth holds the months value (in decimal)
'           : pYear holds the years value (in decimal)
' Output    : None
' Notes     : None
'
$define DS1307_WriteDate(pDayOfWeek, pDay, pMonth, pYear) '
    RTC_DayOfWeek = pDayOfWeek                            '
    RTC_Day = pDay                                        '
    RTC_Month = pMonth                                    '
    RTC_Year = pYear                                      '
    GoSub _DS1307_WriteDate
   
_DS1307_WriteDate: 
    DS1307_DecToBCD(RTC_DayOfWeek, RTC_DayOfWeek)     ' Convert the WeekDay value to decimal
    DS1307_DecToBCD(RTC_Day, RTC_Day)                 ' Convert the Days value to decimal
    DS1307_DecToBCD(RTC_Month, RTC_Month)             ' Convert the Months value to decimal
    DS1307_DecToBCD(RTC_Year, RTC_Year)               ' Convert the Years value to decimal
    
    BusOut cRTC_DEV_ADDR, cRTC_WDAY, [RTC_DayOfWeek, RTC_Day, RTC_Month, RTC_Year]
    Return

'----------------------------------------------------------------------------
' Return the name of the day
' Input     : pDayOfWeek holds the day value 1 to 7 (Sunday to Saturday)
' Output    : RTC_DayName holds a string containing the day's name
' Notes     : None
'
$define DS1307_Get_DayName(pDayOfWeek, pResult) '
    DS1307_bValue = pDayOfWeek                  '
    _DS1307_Get_DayName                         '
    pResult = RTC_DayName

_DS1307_Get_DayName Macro-
    GoSub __DS1307_Get_DayName
Endm

#ifMacro- _DS1307_Get_DayName
__DS1307_Get_DayName:
    Select DS1307_bValue
        Case 1
            RTC_DayName = "Sunday"
        Case 2
            RTC_DayName = "Monday"
        Case 3
            RTC_DayName = "Tuesday"
        Case 4
            RTC_DayName = "Wednesday"
        Case 5
            RTC_DayName = "Thursday"
        Case 6
            RTC_DayName = "Friday"
        Case 7
            RTC_DayName = "Saturday"
    EndSelect
    Return
#endIfMacro

'----------------------------------------------------------------------------
' Return the name of the month
' Input     : pMonth holds the month value 1 to 12 (January to December)
' Output    : RTC_MonthName holds a string containing the month's name
' Notes     : None
'
$define DS1307_Get_MonthName(pMonth, pResult) '
    DS1307_bValue = pMonth                    '
    _DS1307_Get_MonthName                     '
    pResult = RTC_MonthName

_DS1307_Get_MonthName Macro-
    GoSub __DS1307_Get_MonthName
Endm

#ifMacro- _DS1307_Get_MonthName
__DS1307_Get_MonthName:
    Select DS1307_bValue
        Case 1
            RTC_MonthName = "January"
        Case 2
            RTC_MonthName = "February"
        Case 3
            RTC_MonthName = "March"
        Case 4
            RTC_MonthName = "April"
        Case 5
            RTC_MonthName = "May"
        Case 6
            RTC_MonthName = "June"
        Case 7
            RTC_MonthName = "July"
        Case 8
            RTC_MonthName = "August"
        Case 9
            RTC_MonthName = "September"
        Case 10
            RTC_MonthName = "October"
        Case 11
            RTC_MonthName = "November"
        Case 12
            RTC_MonthName = "December"
    EndSelect
    Return
#endIfMacro-

'----------------------------------------------------------------------------
' Detect the DS1307
' Input     : None
' Output    : pResult holds 1 if the DS1307 is available
' Notes     : Uses the Ack return for confirmation
'           : This procedure assumes that there is only 1 I2C device attached to the bus
'
$define DS1307_Present(pResult)             '
     BusIn cRTC_DEV_ADDR, cRTC_TIME,[WREG0] '
     pResult = ~SRbits_C

'----------------------------------------------------------------------------
_DS1307_Main:

$endif  ' _DS1307_INC_


